from Globals import DTMLFile

BAD_CONTROLLER_NAMES = ['layouts']
BAD_ACTION_NAMES = ['init', 'before_filter', 'after_filter']

class Dispatcher(object):
  """Dispatches requests to the appropriate controller+view"""
  
  def __init__(self, container, request, path):
    self.path = path
    self.container = container
    self.request = request
    self.DEFAULT_CONTROLLER = hasattr(self.container, 'default_controller') and self.container.default_controller or 'home'
    self.DEFAULT_ACTION = hasattr(self.container, 'default_action') and self.container.default_action or 'index'
    self.DEFAULT_ID = None
    self.SESSION_KEY = hasattr(self.container, 'session_key') and self.container.session_key or 'react_instance'
  
  def __call__(self, controller=None, action=None, id=None, args={}, return_instance=False):
    """
    The Dispatcher is passed back to Zope, which expects to execute
    the '__call__' method to get the content. Here is where we turn
    controllers and views into a response to be used by Zope.
    """

    # files in the Public folder should be called without any magic
    # note: this will not pass along arguments (is that a problem?)
    if len(self.path) >= 2:
      folder, item = self.path[-2:]
      if folder in self.container.public.objectIds('Folder'):
        obj = self.container.public[folder][item]
        # !!! This needs a lot of work
        try:
          return obj() # file, script
        except:
          try:
            return obj(self.container, self.request) # dtml
          except:
            self.request.RESPONSE.setHeader('content-type', obj.content_type)
            return obj.data # image
    
    # preserve the flash until a View template is rendered
    if self.request.SESSION.has_key(self.SESSION_KEY) and self.request.SESSION[self.SESSION_KEY].has_key('flash'):
      flash = self.request.SESSION[self.SESSION_KEY]['flash']
    else:
      flash = {}
    
    # blow away the session dictionary
    session = self.request.SESSION[self.SESSION_KEY] = {}
    
    # this is a hack for some bug I don't understand
    # (might be unnecessary now that React is a product?)
    if args.has_key('controller'): args = {}
    
    # use the controller, action, and id passed as arguments
    # or the default values
    controller_name = controller or self.DEFAULT_CONTROLLER
    action_name = action or self.DEFAULT_ACTION
    id = id or self.DEFAULT_ID
    
    # depending on the path components given,
    # override the argument-based controller, action, and id
    if len(self.path) >= 1:
      controller_name = self.path[0]
    if len(self.path) >= 2:
      action_name = self.path[1]
    if len(self.path) >= 3:
      id = self.path[2]
    
    # check that someone's not trying to be sneaky
    if controller_name in BAD_CONTROLLER_NAMES:
      controller_name = self.DEFAULT_CONTROLLER
    if action_name in BAD_ACTION_NAMES:
      action_name = self.DEFAULT_ACTION
    
    # prepare the helpers dictionary
    helpers = Helpers(self.container, self.request)._methods()
    
    # prepare the params dictionary
    params = self.request.form
    params['id'] = id
    
    # cobble together the helpers and some other stuff
    # that would be handy as globals inside the controller action
    bindings = helpers.copy()
    bindings['flash'] = flash
    bindings['models'] = self.container.models
    bindings['params'] = params
      
    # get the controller
    try:
      controller_script = getattr(self.container.controllers, controller_name)
      controller_base = controller_script._exec(bindings, [], {}) # returns the class (for now)
      if controller_base == None:
        raise Exception, 'You must return your controller class from its containing Python Script (add "return %s" as the last line of controllers/%s).' % (controller_name, controller_name)
      # do some mumbo-jumbo so the controller methods can actually do stuff
      # I don't think this is needed for Zope 2.7.5+
      class controller(controller_base):
        _guarded_writes = 1
        def __guarded_setitem__(self, key, value):
          if not key.startswith('_'): setattr(self, key, value)
    except AttributeError:
      class controller(object): pass # a generic object class
    
    # set up the instance variable
    instance = controller()
    if hasattr(instance, 'init'):
      instance.init()
    for key in args:
      setattr(instance, key, args[key])
    instance.flash = flash
    
    # store all the parameters in the instance variable
    instance.params = params
    
    # store the controller, action, and id in the instance variable
    session['controller'] = instance.controller = controller_name
    session['action'] = instance.action = action_name
    instance.id = id
    
    resp = None

    # call the before_filter if it exists
    if hasattr(instance, 'before_filter'):
      filter = getattr(instance, 'before_filter')
      resp = filter()
    
    # call the controller method (action) if it exists
    if resp == None and hasattr(instance, action_name):
      action = getattr(instance, action_name)
      resp = action()
      # in case 'action' was changed by render_action()
      action_name = instance.action = session['action']
    
    # call the after_filter if it exists
    if hasattr(instance, 'after_filter'):
      filter = getattr(instance, 'after_filter')
      resp = filter() or resp
    
    # update the flash in the session variable
    self.request.SESSION[self.SESSION_KEY]['flash'] = instance.flash
    
    # if nothing was returned from the action, then render the view template and return it
    if resp == None:
      try:
        view = getattr(getattr(self.container.views, controller_name), action_name)
      except AttributeError:
        if controller_name == self.DEFAULT_CONTROLLER and action_name == self.DEFAULT_ACTION and hasattr(self.container.views, 'react_docs'):
          return helpers['redirect_to'](controller='react_docs', action='index')
        else:
          raise "The view template (%s/%s) was not found. Nothing was returned from the controller action, so the view template is required." % (controller_name, action_name)
      else:
        content = view._text # preserve the content
        try: layout = instance.layout
        except: layout = instance.layout = None
        # let's get ready to pollute the Page Template's namespace!
        kwargs = helpers.copy()
        kwargs['layout'] = layout
        kwargs.update(instance.__dict__)
        # We're forcing the Page Template to compile itself
        # every time its needed, which slows things down a bit.
        # I might fix this someday, but don't hold your breath.
        view._v_cooked = 0
        if layout:
          view._text = \
            '<div metal:use-macro="here/layouts/' + layout + '/macros/layout">' + \
            '<div metal:fill-slot="body">' + \
            content + \
            '</div></div>'
        resp = view.pt_render(extra_context=kwargs)
        if layout: # restore the content
          view._text = content
        # blow away the flash
        if self.request.SESSION.has_key(self.SESSION_KEY) and self.request.SESSION[self.SESSION_KEY].has_key('flash'):
          self.request.SESSION[self.SESSION_KEY]['flash'] = {}
    
    # in case the instance is needed by some other class (namely the mailer)
    if return_instance:
      return resp, instance
    else:
      return resp


class Helpers(object):
  """Holds some useful helpers."""
  
  def __init__(self, container, request):
    self.container = container
    self.request = request
    self.DEFAULT_ACTION = hasattr(self.container, 'default_action') and self.container.default_action or 'index'
    self.SESSION_KEY = hasattr(self.container, 'session_key') and self.container.session_key or 'react_instance'
  
  def _methods(self):
    """
    Gets all these helpers plus the application helpers
    (in the 'helpers' folder) and returns them all as a dictionary
    suitable to be passed to the controller as a bindings dictionary.
    """
    methods = {}
    for method in [m for m in dir(self) if not m.startswith('_')]:
      methods[method] = getattr(self, method)
    if hasattr(self.container, 'helpers'):
      for id in self.container.helpers.objectIds():
        methods[id] = getattr(self.container.helpers, id)
    return methods
  
  def url_for(self, controller=None, action='', id=None, **kwargs):
    """
    This helper script creates a URL from the specified arguments,
    similar in style to the Ruby on Rails url_for helper.
    
    Typical usage:
    url_for(controller='posts', action='list') # no id
    url_for(action='edit', id=instance['id']) # defaults to current controller
    url_for(controller='posts', action='view', id=instance['id'])
    url_for(controller='posts') # default action, no id
    url_for(action='list', extra_arg1='val1', extra_arg2='val2') # current controller, no id, and extra arguments
    
    the 'controller' argument defaults to the current controller
    the 'action' argument defaults to 'default_action' prop on the application root
    the 'id' argument does not have a default; you must specify it if the action will need it
    extra arguments are appended to the url like '?extra_arg1=val1&extra_arg2=val2'
      and available to the action in the instance.params dictionary
    """
    
    instance = self.request.SESSION[self.SESSION_KEY]
    
    if controller == None:
      controller = instance['controller']
    
    controller_action_id = '/%s' % controller
    if action or id != None:
      controller_action_id += '/%s' % (action or self.DEFAULT_ACTION)
    if id != None:
      controller_action_id += '/%s' % id
    
    url = self.container.absolute_url() + controller_action_id.strip()
    
    arg_pairs = []
    for kwarg in kwargs.keys():
      arg_pairs.append('%s=%s' % (kwarg, kwargs[kwarg]))
    
    if arg_pairs:
      url += '?' + '&'.join(arg_pairs)
    
    return url
  
  def redirect_to(self, url=None, controller=None, action=None, id=None, **kwargs):
    """
    Returns a redirect based on the url or controller/action/id/args specified.
    """
    if url == None:
      url = self.url_for(controller, action, id, **kwargs)
    return self.request.RESPONSE.redirect(url)
  
  def link_to(self, link, **kwargs):
    """
    Creates an HTML link. The 'link' argument is the text/markup displayed
    inside the link. The other arguments are passed to the url_for helper
    to construct the url.
    """
    url = self.url_for(**kwargs)
    return '<a href="%s">%s</a>' % (url, link)
  
  def link_to_remote(self, link, update, url):
    """
    Creates an Ajaxed link that calls a controller+view
    and inserts the returned content into the specified container.
    
    'link' is what to display as the link
    'update' is the name of the DIV or other container to insert the content
    'url' is a dictionary of the url_for style, i.e.
      {'controller': 'home', 'action':'update', 'id':3}
    """
    extra_params = {}
    
    if type(url) == type({}):
      base_url = {}
      for key in url.keys():
        if key in ['controller', 'action', 'id']:
          base_url[key] = url[key]
          del(url[key])
      extra_params = url
      url = self.url_for(**base_url)
    
    options = ['asynchronous:true']
    
    if extra_params:
      parameters = []
      for param in extra_params.keys():
        parameters.append("%s=%s" % (param, extra_params[param]))
      parameters = '&'.join(parameters)
      options.append('parameters:\'%s\'' % parameters)
    
    options = '{' + ','.join(options) + '}'
    
    return '<a href="#" onclick="new Ajax.Updater(\'%s\', \'%s\', %s); return false;">%s</a>' % (update, url, options, link)
  
  def send_mail(self, controller, action, id=None, **kwargs):
    """
    Renders a controller action + view and sends it as an email to
    the address specified. The controller action must set at least three
    instance attributes: to_address, from_address, and subject.
    Optionally, the action can set content_type. If not set,
    the content type is guessed based on the specified layout.
    """
    # set up and call the dispatcher to get the rendered page
    dispatcher = Dispatcher(self.container, self.request, [])
    content, instance = dispatcher(controller, action, id, kwargs, return_instance=1)
    if hasattr(instance, 'content_type'):
      content_type = instance.content_type # content_type specified on the instance
    elif hasattr(instance, 'layout') and instance.layout:
      content_type = 'text/html' # not specified, let's assume html because there's a layout
    else:
      content_type = 'text/plain' # not specified, no layout, so probably just text
    # stick the rendered page in the mailer dtml and send the mail
    self.container._mailer(
      self.container,
      self.request,
      content_type = content_type,
      to_address = instance.to_address,
      from_address = instance.from_address,
      subject = instance.subject,
      content = content,
    )
    
  def render_action(self, action):
    """
    Instructs React to render the view template associated with 'action'
    rather than the view template associated with the current action.
    """
    self.request.SESSION[self.SESSION_KEY]['action'] = action
